If($PSVersionTable.PSVersion.Major -lt 4) {
  (new-object -ComObject wscript.shell).Popup("Script requires Powershell V4",0,"OK")
  Exit
}


function Load-Dialog {
  Param(
    [Parameter(Mandatory=$True,Position=1)]
    [string]$XamlPath
  )

  [xml]$Global:xmlWPF = Get-Content -Path $XamlPath

  #Add WPF and Windows Forms assemblies
  try{
    Add-Type -AssemblyName PresentationCore,PresentationFramework,WindowsBase,system.windows.forms
  } catch {
    Throw "Failed to load Windows Presentation Framework assemblies."
  }

  #Create the XAML reader using a new XML node reader
  $Global:xamGUI = [Windows.Markup.XamlReader]::Load((new-object System.Xml.XmlNodeReader $xmlWPF))

  #Create hooks to each named object in the XAML
  $xmlWPF.SelectNodes("//*[@Name]") | %{
    Set-Variable -Name ($_.Name) -Value $xamGUI.FindName($_.Name) -Scope Global
  }
}

function Get-FileName {
  [void][System.Reflection.Assembly]::LoadWithPartialName('System.windows.forms')
   
  $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
  $OpenFileDialog.filter = 'Bookmark Files (*.html)| *.html' # Set the file types visible to dialog
   
  $OpenFileDialog.initialDirectory = ''
  $OpenFileDialog.ShowDialog() | Out-Null
  $OpenFileDialog.filename
}

function Get-Folder {
  Add-Type -AssemblyName System.Windows.Forms
  $FolderBrowser = New-Object System.Windows.Forms.FolderBrowserDialog
  [void]$FolderBrowser.ShowDialog()
  return $FolderBrowser.SelectedPath
}  

Function Convert-UTF {
  Param(
    [String]$SourceFile
  )
  $Buffer = Get-Content $SourceFile -Encoding byte
  
#  $Encoding = [System.Text.Encoding]::GetEncoding("gb2312")
  $Encoding = [Console]::OutputEncoding  
  $String = $Encoding.GetString($Buffer)
  Out-File -FilePath .\C2Htmp.txt -Encoding utf8 -InputObject $String
}

function Collect-CGP {
  $Label3.Content = 'Select folder with CGP files'
  $sFolder = Get-Folder
  If($sFolder -eq '') { Return }
  $aFiles = (Get-ChildItem -Include *.cgp -Path ($sFolder + "\*"))
  If(($sFolder.LastIndexOf('\') + 1) -eq $sFolder.Length) {
  $sOutfile =  ($sFolder | % { $_.Substring(0, $_.LastIndexOf('\') + 1) }) `
                + 'CGP2HTML-' + ("{0:yyyy}" -f (Get-Date)) + ("{0:MM}" -f (Get-Date)) `
                + ("{0:dd}" -f (Get-Date)) + ('{0:hh}' -f (Get-Date)) `
                + ('{0:mm}' -f (Get-Date)) + ('{0:ss}' -f (Get-Date)) + '.html'
  } else {
  $sOutfile =  $sFolder + '\' + 'CGP2HTML-' + ("{0:yyyy}" -f (Get-Date)) `
                + ("{0:MM}" -f (Get-Date)) + ("{0:dd}" -f (Get-Date)) `
                + ('{0:hh}' -f (Get-Date)) + ('{0:mm}' -f (Get-Date)) `
                + ('{0:ss}' -f (Get-Date)) + '.html'
  }
  Write-Header $sOutfile $sFolder
  Convert-CGP $aFiles $sOutfile
  Write-Host ''
  $Label3.Content = $sOutfile
}

Function Write-Header {
  Param(
    [String]$OutFile,
    [String]$Folder
  )
  '<!DOCTYPE NETSCAPE-Bookmark-file-1>' | Out-File -Encoding utf8 -FilePath $OutFile -NoClobber
  '<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<TITLE>Bookmarks</TITLE>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<H1>Bookmarks</H1>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DT><H3>{0} --- {1}</H3>' -f $Folder, (Get-Date) | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
}

Function Convert-CGP {
  Param(
    [Object]$File,
    [String]$Outfile
  )
  for($i=0; $i -lt $File.Length; $i++) {
    $tmpFile = $File[$i] | % { $_.BaseName }
    $Label3.Content = "Converting: " + $File[$i]
    Convert-UTF $File[$i]
    '<DT><H3>' + $tmpFile + '</H3></DT>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    $Lines = (Get-Content ".\C2Htmp.txt")
    for($j=2; $j -lt $Lines.Length; $j+=2) {
      '<DT><A HREF="{0}">{1}</A>' -f $Lines[$j].Substring($Lines[$j].IndexOf("=") + 1), `
                                     $Lines[$j - 1].Substring($Lines[$j - 1].IndexOf("=") + 1) `
                                     | Out-File -Encoding utf8 -FilePath $OutFile -Append
    }
    '</DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    Remove-Item '.\C2HTmp.txt'
  }
  '</DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
}

function Convert-HTML {
  $OutputEncoding = [Console]::OutputEncoding
  $done = ''
  $Label3.Content = "Select Mozilla/Chrome bookmark file"
  $filename = Get-FileName
  if ($filename -eq '') {
    return
  }
  else
  {
    $CGPfile = Get-Content $filename -Encoding UTF8
    $Label3.Content = "Select output folder for Group files"
    $destFolder = Get-Folder
    $Label3.Content = $destFolder
  }
  
  for ($i = 0; $i -lt $CGPfile.Length; $i++) {
    if ($CGPfile[$i].Contains('<H3')) {
      $j = $i
    }
    if ($CGPfile[$i].Contains('<DT><A HREF=')) {
      $k++
      $CGPfile[$j] -match '<H3.*?>(.+)</H.*'
      $temp = $Matches[1]
      if ($destFolder -match '.*\\$') {
        $Outfile = $destFolder + $temp + '.cgp'
      } else {
        $Outfile = $destFolder + '\' + $temp + '.cgp'
      }

      if ($CGPfile[$i] -match '.+HREF=\"(.+?)\".*?>(.+)<.+') {
        $url = $Matches[1]
        $name = $Matches[2]
        if ($done -ne $Outfile) {
          $k = 0
          '[Group]' | Out-File -Encoding default -FilePath $Outfile
          $done = $Outfile
          $Label3.Content = 'Group being generated: {0}' -f $done
          'name{0}={1}' -f $k, $name | Out-File -Encoding default -FilePath $Outfile -Append
          'url{0}={1}' -f $k, $url | Out-File -Encoding default -FilePath $Outfile -Append
        } else {
          'name{0}={1}' -f $k, $name | Out-File -Encoding default -FilePath $Outfile -Append
          'url{0}={1}' -f $k, $url | Out-File -Encoding default -FilePath $Outfile -Append
        }
      }
    }
  }
  $Label3.Content = 'Group files path - {0}' -f $destFolder
}

#Required to load the XAML form and create the PowerShell Variables
Load-Dialog -XamlPath '.\HTML2CGP.xaml'

#EVENT Handlers
$Button1.add_Click({
  Collect-CGP
})


$Button2.add_Click({
  Convert-HTML
})

#Launch the window
$xamGUI.ShowDialog() | out-null
